CMAKE_MINIMUM_REQUIRED(VERSION 3.0)

INCLUDE(ExternalProject)


FUNCTION(BUILD_CLN BUILD_DIR INSTALL_PREFIX)

  SET(CLN_SRC ${BUILD_DIR}/src)
  SET(CLN_BIN ${BUILD_DIR}/bin)
  SET(CLN_INSTALL ${INSTALL_PREFIX})
  SET(CLN_INSTALL ${INSTALL_PREFIX} PARENT_SCOPE)

  EXTERNALPROJECT_ADD(
    CLN
    GIT_REPOSITORY git://www.ginac.de/cln.git
    GIT_TAG 38d517624d3bc6379c7e5c8ac014fdff3d2d3f53

    SOURCE_DIR ${CLN_SRC}
    BINARY_DIR ${CLN_BIN}

    CONFIGURE_COMMAND ${CLN_SRC}/configure --quiet --prefix=${CLN_INSTALL}
    BUILD_COMMAND make
    INSTALL_COMMAND make install
  )

  EXTERNALPROJECT_ADD_STEP(
    CLN
    autoconf

    COMMAND autoreconf -i ${CLN_SRC}
    DEPENDEES download
    DEPENDERS update
  )

  SET(CLN_FOUND PARENT_SCOPE)

  SET(CLN_INCLUDE_DIR ${CLN_INSTALL}/include)
  SET(CLN_INCLUDE_DIR ${CLN_INCLUDE_DIR} PARENT_SCOPE)
  SET(CLN_INCLUDE_DIRS ${CLN_INCLUDE_DIR} PARENT_SCOPE)

  SET(CLN_LIBRARY ${CLN_INSTALL}/lib/${CMAKE_FIND_LIBRARY_PREFIXES}cln${CMAKE_STATIC_LIBRARY_SUFFIX})
  SET(CLN_LIBRARY ${CLN_LIBRARY} PARENT_SCOPE)
  SET(CLN_LIBRARIES ${CLN_LIBRARY} PARENT_SCOPE)

  SET(CLN_LIBRARY_DIR ${CLN_INSTALL}/lib)
  SET(CLN_LIBRARY_DIR ${CLN_LIBRARY_DIR} PARENT_SCOPE)
  SET(CLN_LIBRARY_DIRS ${CLN_LIBRARY_DIR} PARENT_SCOPE)

  ADD_DEPENDENCIES(DEPS CLN)

ENDFUNCTION(BUILD_CLN)


FUNCTION(BUILD_GINAC BUILD_DIR CLN_INSTALL_DIR CLN_INCLUDE_DIRS CLN_LIBRARY_DIRS CLN_LIBRARIES INSTALL_PREFIX)

  SET(GINAC_SRC ${BUILD_DIR}/src)
  SET(GINAC_BIN ${BUILD_DIR}/bin)
  SET(GINAC_INSTALL ${INSTALL_PREFIX})
  SET(GINAC_INSTALL ${INSTALL_PREFIX} PARENT_SCOPE)

  SET(ENV{CLN_DIR} ${CLN_INSTALL_DIR})

  EXTERNALPROJECT_ADD(
    GiNaC
    DEPENDS CLN
    GIT_REPOSITORY git://www.ginac.de/ginac.git
    GIT_TAG release_1-7-3

    SOURCE_DIR ${GINAC_SRC}
    BINARY_DIR ${GINAC_BIN}

    CMAKE_ARGS -DCMAKE_BUILD_TYPE=Release -DBUILD_SHARED_LIBS=FALSE -DCMAKE_INSTALL_PREFIX=${GINAC_INSTALL} -DCMAKE_C_COMPILER=${CMAKE_C_COMPILER} -DCMAKE_CXX_COMPILER=${CMAKE_CXX_COMPILER} -DCMAKE_C_FLAGS_DEBUG=${CMAKE_C_FLAGS_DEBUG} -DCMAKE_CXX_FLAGS_DEBUG=${CMAKE_CXX_FLAGS_DEBUG} -DCMAKE_C_FLAGS_RELEASE=${CMAKE_C_FLAGS_RELEASE} -DCMAKE_CXX_FLAGS_RELEASE=${CMAKE_CXX_FLAGS_RELEASE}
  )

  SET(GINAC_FOUND PARENT_SCOPE)

  SET(GINAC_INCLUDE_DIR ${GINAC_INSTALL}/include)
  SET(GINAC_INCLUDE_DIR ${GINAC_INCLUDE_DIR} PARENT_SCOPE)
  SET(GINAC_INCLUDE_DIRS ${GINAC_INCLUDE_DIR} ${CLN_INCLUDE_DIRS} PARENT_SCOPE)

  SET(GINAC_LIBRARY ${GINAC_INSTALL}/lib/${CMAKE_FIND_LIBRARY_PREFIXES}ginac${CMAKE_STATIC_LIBRARY_SUFFIX})
  SET(GINAC_LIBRARY ${GINAC_LIBRARY} PARENT_SCOPE)
  SET(GINAC_LIBRARIES ${GINAC_LIBRARY} ${CLN_LIBRARIES} PARENT_SCOPE)

  SET(GINAC_LIBRARY_DIR ${GINAC_INSTALL}/lib)
  SET(GINAC_LIBRARY_DIR ${GINAC_LIBRARY_DIR} PARENT_SCOPE)
  SET(GINAC_LIBRARY_DIRS ${GINAC_LIBRARY_DIR} ${CLN_LIBRARY_DIRS})

  ADD_DEPENDENCIES(DEPS GiNaC)

ENDFUNCTION(BUILD_GINAC)
